<?php

namespace DMG\Enhanced_Permalinks;

if ( ! class_exists( 'DMG\Enhanced_Permalinks\Load' ) ) :

	/**
	 * initialize the Enhanced Permalinks features
	 *
	 * Class Load
	 *
	 * @package DMG\Enhanced_Permalinks
	 */
	class Load {

		const PLUGIN_VERSION = '1.1.1';

		/**
		 * Class constructor.
		 */
		public function __construct() {
			require_once( __DIR__ . '/class-admin.php' );
			require_once( __DIR__ . '/class-filters.php' );
			require_once( __DIR__ . '/class-api.php' );

			// Load js assets
			add_action( 'current_screen', [ __CLASS__, 'current_screen' ] );

			// Add a custom column for the quick edit view
			add_filter( 'manage_posts_columns', [ __CLASS__, 'add_custom_column' ], 10, 9 );
			add_action( 'manage_posts_custom_column', [ __CLASS__, 'quick_edit_fields' ], 10, 2 );
		}

		/**
		 * Check current screen to determine the js/styles to enqueue.
		 *
		 * @param WP_Screen $screen The screen object.
		 */
		public static function current_screen( $screen ) {
			/**
			 * Filter the list of post types enabled
			 *
			 * @param array The array of enabled post types
			 */
			$enabled_post_types = apply_filters( 'dmg_enhanced_permalinks_enabled_post_types', [ 'post' ] );

			if ( ! is_array( $enabled_post_types ) ) {
				return;
			}

			if ( ! in_array( $screen->post_type, $enabled_post_types, true ) ) {
				return;
			}

			if ( ! empty( $screen->base ) && 'post' === $screen->base ) {
				// Check if block editor is active and enqueue assets accordingly.
				if ( method_exists( $screen, 'is_block_editor' ) && $screen->is_block_editor() ) {
					add_action( 'enqueue_block_editor_assets', [ __CLASS__, 'enqueue_block_editor_assets' ] );
				}
			} elseif ( 'edit-post' === $screen->id ) {
				add_action( 'admin_enqueue_scripts', [ __CLASS__, 'enqueue_quick_edit_assets' ] );
			}
		}

		/**
		 * Enqueue scripts for the Gutenberg screen.
		 */
		public static function enqueue_block_editor_assets() {
			wp_enqueue_script(
				'enhanced-permalinks',
				plugin_dir_url( __DIR__ ) . 'assets/js/enhanced-permalinks.js',
				[],
				self::PLUGIN_VERSION
			);
		}

		/**
		 * Enqueue scripts/css for Quick Edit screen.
		 */
		public static function enqueue_quick_edit_assets() {
			wp_enqueue_script(
				'enhanced-permalinks-quick-edit',
				plugin_dir_url( __DIR__ ) . 'assets/js/quick-edit.js',
				[],
				self::PLUGIN_VERSION
			);

			wp_enqueue_style(
				'enhanced-permalinks-quick-edit-css',
				plugin_dir_url( __DIR__ ) . 'assets/css/quick-edit-styles.css',
				[],
				self::PLUGIN_VERSION
			);
		}

		/**
		 * Adds a column
		 *
		 * @param $defaults
		 * @param $post_type
		 *
		 * @return mixed
		 */
		public static function add_custom_column( $defaults, $post_type ) {
			if ( $post_type === 'post' ) {
				$defaults['post-status'] = '';
			}

			return $defaults;
		}

		/**
		 * Retrieve the current post status to populate the Quick Edit form with the original post status
		 *
		 * @param string $column The column name.
		 * @param int $post_id The post ID.
		 */
		public static function quick_edit_fields( $column, $post_id ) {
			switch ( $column ) {
				case 'post-status':
					printf(
						'<div class="hidden original_qe_post_status">%s</div>',
						esc_html( get_post_status( $post_id ) )
					);
					break;
			}
		}
	}

	new Load();

endif;
