<?php

namespace DMG\Enhanced_Permalinks;

/**
 * Class Filters
 *
 * Adds all the necessary hooks and utilities for filtering the permalink.
 *
 * @package DMG\Enhanced_Permalinks
 */
class Filters {

	/**
	 * Class constructor.
	 */
	public function __construct() {
		require_once( __DIR__ . '/filters/class-stop-words.php' );
		require_once( __DIR__ . '/filters/class-custom-source.php' );

		add_filter( 'wp_insert_post_data', [ __CLASS__, 'filter_permalink' ], 99, 2 );
		add_filter( 'enhanced_permalinks_post_name', [ __CLASS__, 'return_the_slug' ], 10, 2 );
	}

	/**
	 * Filter permalink on non-public articles (quick edit screen)
	 *
	 * @param $data
	 * @param $postarr
	 *
	 * @return mixed
	 */
	public static function filter_permalink( $data, $postarr ) {
		if ( isset ( $postarr['original_qe_post_status'] ) && 'publish' !== $postarr['original_qe_post_status'] ) {
			$data['post_name'] = apply_filters( 'enhanced_permalinks_post_name', $data['post_title'], $postarr );
		}

		return $data;
	}

	/**
	 * Return the filtered slug
	 *
	 * @param $slug
	 * @param $postarr
	 *
	 * @return string
	 */
	public static function return_the_slug( $slug, $postarr ) {

		$slug_source = apply_filters( 'enhanced_permalinks_source', $slug, $postarr );

		$slug_source = self::deburr( trim( $slug_source ) );

		$slug_parts = explode( ' ', stripslashes( $slug_source ) );

		$filtered_slug_parts = apply_filters( 'enhanced_permalinks_filter_slug', $slug_parts );

		$cleaned_slug = join( ' ', $filtered_slug_parts );

		$slug = self::clean_for_slug( $cleaned_slug );

		return $slug;
	}

	/**
	 * Replicate the gutenberg cleanForSlug replacements
	 *
	 * @param int $post_id
	 * @param string $slug_string
	 *
	 * @return string New permalink
	 */
	public static function clean_for_slug( $slug_string ) {

		// Sanitize the slug
		$slug = urldecode( sanitize_title( $slug_string ) );

		// Add these steps to replicate the gutenberg `cleanForSlug` util behaviour
		$slug = strtolower( self::deburr( trim( $slug ) ) );

		return sanitize_title( $slug );
	}

	/**
	 * Translate the deburr function from JavaScript to PHP
	 *
	 * @param $str
	 *
	 * @return false|string|string[]|null
	 */
	public static function deburr( $str ) {

		// Regular expression to match common currency symbols
		$currency_symbols = '/[$€£¥₹]/u'; // Add more currency symbols if needed

		// Transliterate characters to their closest ASCII representation stripping off currencies
		$str = iconv( 'UTF-8', 'ASCII//TRANSLIT', preg_replace($currency_symbols, '', $str) );

		// Remove any remaining non-ASCII characters.
		$str = preg_replace( '/[^\x20-\x7E]/', '', $str );

		// Strip everything except letters, numbers, hyphens, underscores, apostrophes and spaces
		$str = preg_replace('/[^a-zA-Z0-9\-_\'\s]/', ' ', stripslashes( $str ) );

		return $str;
	}
}

new Filters();
