<?php

namespace DMG\Enhanced_Permalinks;

/**
 * Class Stop_Words
 *
 * Add some filters to remove unwanted words from it to keep the URLs concise
 *
 */
class Stop_Words {

	/**
	 * Option name
	 *
	 * @var string
	 */
	CONST OPTION_NAME = 'enhanced_permalinks_stop_words';

	/**
	 * Initial load.
	 */
	public function __construct() {
		add_action( 'enhanced_permalinks_settings', [ __CLASS__, 'register_settings' ], 10 );
		add_filter( 'enhanced_permalinks_filter_slug', [ __CLASS__, 'strip_unwanted_words' ], 10, 1 );
	}


	/**
	 * Get the stop words.
	 *
	 * @return array The array of stop words.
	 */
	public static function get_stop_words() {
		$option = get_option( self::OPTION_NAME );

		return explode( ',', strtolower( $option ) );
	}

	/**
	 * Strip unwanted words from an array of strings
	 *
	 * @param $slug_parts
	 *
	 * @return array
	 */
	public static function strip_unwanted_words( $slug_parts ) {
		/**
		 * Filter to modify the list of words to remove defined in settings
		 *
		 * @param array The array of stop words to remove from the URL.
		 */
		$stop_words = apply_filters( 'dmg_permalink_stop_words', self::get_stop_words() );

		// Strip off the unwanted words.
		$filtered_slug_parts = array_diff( $slug_parts, $stop_words );

		// Apply the filter if there more than 2 words left.
		if ( count( $filtered_slug_parts ) > 2 ) {
			$slug_parts = $filtered_slug_parts;
		}

		return $slug_parts;
	}

	/**
	 * Register Settings for the module
	 *
	 * @param $settings_page
	 */
	public static function register_settings( $settings_page ) {
		// Add the settings field to the "optional" Unwanted Words section on Permalink Settings
		add_settings_field(
			self::OPTION_NAME,
			sprintf(
				'<label for="%s">%s</label>',
				esc_attr( self::OPTION_NAME ),
				__( 'Words to remove from permalinks', 'permalink_stop_words' )
			),
			[ __CLASS__, 'setting_textarea_field_html' ],
			$settings_page,
			'enhanced_permalinks'
		);

		// Register a general setting
		register_setting(
			$settings_page,
			self::OPTION_NAME,
			'sanitize_text_field'
		);
	}

	/**
	 * Outputs the settings textarea field HTML.
	 */
	public static function setting_textarea_field_html() {
		printf(
			'<textarea id="%s" name="%s" rows="8" cols="50">%s</textarea><br />
				<small class="admin-note">Comma-separate list of unwanted words. e.g (a,I\'d,I’d,about,above,after,again,against,what’s)</small>',
			esc_attr( self::OPTION_NAME ),
			esc_attr( self::OPTION_NAME ),
			esc_textarea( get_option( self::OPTION_NAME ) )
		);
	}


}

new Stop_Words();
